/*
    Part of X2GoAdminCenter.
    Copyright (C) 2011-2015 Oleksandr Shneyder<oleksandr.shneyder@obviously-nice.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "x2gosessionadminwindow.h"
#include <QTimer>
#include <QPixmap>
#include <QInputDialog>
#include <QDebug>
#include <QMessageBox>
#include <centerinterface.h>
#include <QMenu>
#include <QHeaderView>

void X2GoSessionAdminWindow::installTranslator()
{
    QTranslator* trans=new QTranslator();
    QString filename=QString ( ":/x2gosessionadmingui_%1" ).arg (
                         QLocale::system().name() );
    filename=filename.toLower();
    if ( !trans->load ( filename ) )
    {
        qDebug ( "Can't load translator (%s) !\n",
                 filename.toLocal8Bit().data() );
    }
    else
    {
        QCoreApplication::installTranslator ( trans );
    }


    QTranslator* qtTranslator=new QTranslator;
    filename=QString ( ":/qt_%1" ).arg ( QLocale::system().name() );
    if ( !qtTranslator->load ( filename ) )
    {
        qDebug()<< "Can't load translator "<<
                filename.toLocal8Bit().data() ;
    }
    else
    {
        QCoreApplication::installTranslator ( qtTranslator );
    }
}

X2GoSessionAdminWindow::X2GoSessionAdminWindow()
{
    installTranslator();
    NAME =tr("X2Go session administration");
    DESCRIPTION= tr("Use this module to administrate X2Go sessions");

    DEVELOPERS= "Oleksandr Shneyder &lt;oleksandr.shneyder@obviously-nice.de&gt;<br>"
                "Heinz-Markus Graesing &lt;heinz-m.graesing@obviously-nice.de&gt;";

    TRANSLATORS ="Heinz-Markus Graesing &lt;heinz-m.graesing@obviously-nice.de&gt;";

    ABOUT= "<b>"+NAME + "-" + VERSION + "</b><br><br>"
           +DESCRIPTION+"<br><br><b>"
           + tr("Developers:")+"</b><br>" + DEVELOPERS+ "<br><br><b>"
           + tr("Translators:")+"</b><br>"+ TRANSLATORS;


    qDebug()<<"session admin constructor";
    hidden=true;
    setupUi ( this );
    centerInstance=0l;
    treeServers->setAllColumnsShowFocus ( true );
    treeServers->setAlternatingRowColors ( true );
    treeServers->setUniformRowHeights ( true );
    treeServers->setIconSize(QSize(32,32));
}

X2GoSessionAdminWindow::~X2GoSessionAdminWindow()
{

}

void X2GoSessionAdminWindow::startPlugin()
{

    changed=false;
    hidden=false;
    closing=false;
    adminMode=true;

    rpc= new MaiaXmlRpcClient(QUrl(centerInstance->getUrl()),this);

    QSslConfiguration config=rpc->sslConfiguration();
    config.setProtocol(QSsl::AnyProtocol);
    rpc->setSslConfiguration(config);

    connect(rpc, SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)), this,
            SLOT(slotHandleSslErrors(QNetworkReply*,QList<QSslError>)));
    treeServers->setRootIsDecorated(false);
    treeServers->setItemsExpandable(false);

    pbSuspend->setEnabled(false);
    pbTerminate->setEnabled(false);
    treeServers->header()->setResizeMode ( QHeaderView::ResizeToContents );
    treeServers->header()->setSortIndicator ( 1,Qt::AscendingOrder );
    slotGetSessions();
}

void X2GoSessionAdminWindow::slotSuspend()
{
    servTimer.stop();
    QTreeWidgetItem* sel = treeServers->currentItem();
    QString server=sel->text(COL_SERVER);
    QString id=sel->text(COL_ID);
    QVariantList args;
    args<<centerInstance->getLogin()<<centerInstance->getPassword()<<server<<id;
    centerInstance->setStatus(tr("Retrieving information from server, please wait..."));
    rpc->call("x2goadmin.suspendSession",args, this, SLOT(slotSessionsList(QVariant&)),
              this, SLOT(slotRpcFault(int, const QString&)));

}

void X2GoSessionAdminWindow::slotTerminate()
{
    servTimer.stop();
    QTreeWidgetItem* sel = treeServers->currentItem();
    QString server=sel->text(COL_SERVER);
    QString id=sel->text(COL_ID);
    QVariantList args;
    args<<centerInstance->getLogin()<<centerInstance->getPassword()<<server<<id;
    centerInstance->setStatus(tr("Retrieving information from server, please wait..."));
    rpc->call("x2goadmin.terminateSession",args, this, SLOT(slotSessionsList(QVariant&)),
              this, SLOT(slotRpcFault(int, const QString&)));

}

void X2GoSessionAdminWindow::slotRetControl(QVariant& result)
{
    if (hidden)
        return;
    QString res=result.toString();
    qDebug()<<res;
    if (!checkResult(res))
    {
        centerInstance->backToList();
        return;
    }
}

void X2GoSessionAdminWindow::slotGetSessions()
{
    QVariantList args;
    args<<centerInstance->getLogin()<<centerInstance->getPassword();
    rpc->call("x2goadmin.getSessions",args, this, SLOT(slotSessionsList(QVariant&)),
              this, SLOT(slotRpcFault(int, const QString&)));
    centerInstance->setStatus(tr("Retrieving information from server, please wait..."));
    changed=false;
}


void X2GoSessionAdminWindow::slotSelectSession(QTreeWidgetItem* item, int )
{
    pbSuspend->setEnabled(false);
    pbTerminate->setEnabled(false);
    if(!item)
    {
        selectedSession=QString::null;
        return;
    }
    selectedSession=item->text(COL_ID);
    if(item->text(COL_STATE)==tr("Running"))
    {
        pbSuspend->setEnabled(true);
        pbTerminate->setEnabled(true);
    }
    if(item->text(COL_STATE)==tr("Suspended"))
    {
        pbTerminate->setEnabled(true);
    }
}


void X2GoSessionAdminWindow::slotSessionsList(QVariant& result)
{
    if (hidden)
        return;
    QString res=result.toString();
    if (!checkResult(res))
    {
        centerInstance->backToList();
        return;
    }
    centerInstance->setStatus(tr("Ready"));

    treeServers->clear();

    QStringList sessions=result.toString().split("\n",QString::SkipEmptyParts);
    QString session;
    foreach(session, sessions)
    {
        QStringList details=session.split("|");
        QTreeWidgetItem *it=new QTreeWidgetItem(treeServers);
        it->setText(COL_USER, details[7]);
        QString state;
        if(details[3]=="S")
            state=tr("Suspended");
        if(details[3]=="R")
            state=tr("Running");
        if(details[3]=="F")
            state=tr("Finished");
        it->setText(COL_STATE, state);
        it->setText(COL_SERVER, details[2]);
        QDateTime dt=QDateTime::fromString(details[4],Qt::ISODate );
        it->setText(COL_START, dt.toString("dd.MM HH:mm"));
        it->setText(COL_ID, details[0]);
        it->setText(COL_DISPLAY, details[1]);
        it->setText(COL_IP, details[5]);
        if(details[3]=="S")
        {
            dt=QDateTime::fromString(details[6],Qt::ISODate );
            it->setText(COL_INTERRUPTED, dt.toString("dd.MM HH:mm"));
        }
        else
        {
            it->setText(COL_INTERRUPTED, "");
        }
        if(details[0]==selectedSession)
        {
            it->setSelected(true);
            treeServers->setCurrentItem(it);
            slotSelectSession(it,0);
        }
    }

    changed=false;
    servTimer.singleShot(5000, this, SLOT(slotGetSessions()));
}



bool X2GoSessionAdminWindow::closePlugin()
{
    if (closing)
        return false;
    hidden=true;
    return true;
}


void X2GoSessionAdminWindow::setCenterInstance(CenterInterface* obj)
{
    centerInstance=obj;
}


void X2GoSessionAdminWindow::slotHandleSslErrors(QNetworkReply* reply, const QList< QSslError >& errors)
{
    if (hidden)
        return;
    centerInstance->handleSslErrors(reply, errors);
}


void X2GoSessionAdminWindow::slotRpcFault(int, const QString& errMessage)
{
    if (hidden)
        return;
    QMessageBox::critical(this, tr("Connection error"), errMessage);
    centerInstance->backToList();
}


bool X2GoSessionAdminWindow::checkResult(const QString& result)
{
    if (result.indexOf("SERVERDOWN")==0)
    {
        QString res=result;
        res.replace("SERVERDOWN ","" );
        QString errorMessage=tr("Server connection failed<br>Error message:")+"\n"+res;
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    if (result.indexOf("NOACCESS")==0)
    {
        QString res=result;
        res.replace("NOACCESS ","" );
        QString errorMessage=tr("Login failed");
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    if (result.indexOf("CMDERR")==0)
    {
        QString res=result;
        res.replace("CMDERR ","" );
        QString errorMessage=tr("Execution of command failed<br>Error message:")+"\n"+res;
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    if (result.indexOf("DBERR")==0)
    {
        QString res=result;
        res.replace("DBERR ","" );
        QString errorMessage=tr("Database error<br>Error message:")+"\n"+res;
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    return true;
}



Q_EXPORT_PLUGIN2(x2gosessionadmin, X2GoSessionAdminWindow)
