/*
    Part of X2GoAdminCenter.
    Copyright (C) 2011-2015 Oleksandr Shneyder<oleksandr.shneyder@obviously-nice.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "x2gouseradminwindow.h"
#include <QTimer>
#include <QPixmap>
#include <QInputDialog>
#include <QDebug>
#include <QMessageBox>
#include <centerinterface.h>



void X2GouserAdminWindow::installTranslator()
{
    QTranslator* trans=new QTranslator();
    QString filename=QString ( ":/x2gouseradmingui_%1" ).arg (
                         QLocale::system().name() );
    filename=filename.toLower();
    if ( !trans->load ( filename ) )
    {
        qDebug ( "Can't load translator (%s) !\n",
                 filename.toLocal8Bit().data() );
    }
    else
    {
        QCoreApplication::installTranslator ( trans );
    }


    QTranslator* qtTranslator=new QTranslator;
    filename=QString ( ":/qt_%1" ).arg ( QLocale::system().name() );
    if ( !qtTranslator->load ( filename ) )
    {
        qDebug()<< "Can't load translator "<<
                filename.toLocal8Bit().data() ;
    }
    else
    {
        QCoreApplication::installTranslator ( qtTranslator );
    }
}


X2GouserAdminWindow::X2GouserAdminWindow()
{

    installTranslator();

    NAME =tr("X2Go user administration");
    DESCRIPTION= tr("Add, remove and edit users ...");

    DEVELOPERS= "Oleksandr Shneyder &lt;oleksandr.shneyder@obviously-nice.de&gt;<br>"
                "Heinz-Markus Graesing &lt;heinz-m.graesing@obviously-nice.de&gt;";

    TRANSLATORS ="Heinz-Markus Graesing &lt;heinz-m.graesing@obviously-nice.de&gt;";

    ABOUT= "<b>"+NAME + "-" + VERSION + "</b><br><br>"
           +DESCRIPTION+"<br><br><b>"
           + tr("Developers:")+"</b><br>" + DEVELOPERS+ "<br><br><b>"
           + tr("Translators:")+"</b><br>"+ TRANSLATORS;




    qDebug()<<"user admin constructor";


    hidden=true;
    setupUi ( this );
    treeWidget->headerItem()->setText ( COL_FOTO,"");
    pbAddGroup->setIcon ( QPixmap ( ":/png/leftarrow.png" ) );
    pbRemoveGroup->setIcon ( QPixmap ( ":/png/rightarrow.png" ) );
    pbAddUser->setIcon ( QPixmap ( ":/png/add_user.png" ) );
    pbDelUser->setIcon ( QPixmap ( ":/png/delete_user.png" ) );

    pbUserPic->setIcon ( QPixmap ( ":/png/user.png" ) );
    treeWidget->setAllColumnsShowFocus ( true );
    treeWidget->setAlternatingRowColors ( true );
    treeWidget->setUniformRowHeights ( true );
    treeWidget->setIconSize(QSize(32,32));


    setUserViewEnabled(false);
    gbAllUsers->setEnabled(false);
    centerInstance=0l;
}

X2GouserAdminWindow::~X2GouserAdminWindow()
{

}

void X2GouserAdminWindow::startPlugin()
{
    lLogin->hide();
    eLogin->hide();
    eFindLogin->clear();
    eFindName->clear();
    eFname->clear();
    eLname->clear();
    dateBirthday->setDate(QDate());
    spUId->setValue(0);
    eHome->clear();
    eShell->clear();
    lselGroup->clear();
    lunselGroup->clear();
    changed=false;
    adminMode=true;/*centerInstance->adminMode();*/

    QList<int> sizes;
    sizes<<(gbAllUsers->width()+gbUser->width())/2<<sizes[0];
    splitter->setSizes(sizes);
    changed=false;
    hidden=false;
    pbApply->setEnabled(false);
    closing=false;
    lUsername->setText("");
    rpc= new MaiaXmlRpcClient(QUrl(centerInstance->getUrl()),this);

    QSslConfiguration config=rpc->sslConfiguration();
    config.setProtocol(QSsl::AnyProtocol);
    rpc->setSslConfiguration(config);

    connect(rpc, SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)), this,
            SLOT(slotHandleSslErrors(QNetworkReply*,QList<QSslError>)));


    if (!adminMode)
    {
        slotUserSelected(0,0);
        return;
    }
    QSettings* st=centerInstance->getSettings(pluginName());
    cbTree->setChecked(st->value("treeview",false).toBool());
    setUserViewVisible(false);
    slotGetUsers();
}

void X2GouserAdminWindow::slotGetUsers()
{
    QVariantList args;
    args<<centerInstance->getLogin()<<centerInstance->getPassword();
    gbAllUsers->setEnabled(false);
    treeWidget->clear();
    QSettings* st=centerInstance->getSettings(pluginName());
    st->setValue("treeview",cbTree->isChecked());
    st->sync();
    setUserViewVisible(false);


    if (!cbTree->isChecked())
    {
        rpc->call("x2goadmin.getUsers",args, this, SLOT(slotUsersList(QVariant&)),
                  this, SLOT(slotRpcFault(int, const QString&)));
    }
    else
    {
        rpc->call("x2goadmin.getGroupsWithUsers",args, this, SLOT(slotUsersList(QVariant&)),
                  this, SLOT(slotRpcFault(int, const QString&)));
    }
    centerInstance->setStatus(tr("Retrieving information from server, please wait..."));
}


int X2GouserAdminWindow::applyQestion()
{
    return QMessageBox::question(this,tr("Apply changes"),tr("Do you want to apply changes?"),
                                 QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel);
}


bool X2GouserAdminWindow::closePlugin()
{
    if (closing)
        return false;
    if (changed)
    {
        int res=applyQestion();
        if (res==QMessageBox::Cancel)
            return false;
        if (res==QMessageBox::Yes)
        {
            closing=true;
            slotApply();
            return false;
        }
    }
    hidden=true;
    return true;
}


void X2GouserAdminWindow::setCenterInstance(CenterInterface* obj)
{
    centerInstance=obj;
}


void X2GouserAdminWindow::slotHandleSslErrors(QNetworkReply* reply, const QList< QSslError >& errors)
{
    if (hidden)
        return;
    centerInstance->handleSslErrors(reply, errors);
}

void X2GouserAdminWindow::slotRpcFault(int, const QString& errMessage)
{
    if (hidden)
        return;
    qDebug()<<"connection error";
    QMessageBox::critical(this, tr("Connection error"), errMessage);
    centerInstance->backToList();
}

bool X2GouserAdminWindow::checkResult(const QString& result)
{
//     qDebug()<<result;
    if (result.indexOf("SERVERDOWN")==0)
    {
        QString res=result;
        res.replace("SERVERDOWN ","" );
        QString errorMessage=tr("Server connection failed<br>Error message:")+"\n"+res;
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    if (result.indexOf("NOACCESS")==0)
    {
        QString res=result;
        res.replace("NOACCESS ","" );
        QString errorMessage=tr("Login failed:")+"\n"+res;
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    if (result.indexOf("CMDERR")==0)
    {
        QString res=result;
        res.replace("CMDERR ","" );
        QString errorMessage=tr("Execution of command failed<br>Error message:")+"\n"+res;
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    return true;
}

void X2GouserAdminWindow::initUserAttr(const QString& attrString)
{
    for (int i=0; i< USERATTR; ++i)
    {
        userAttributes[i].editable=false;
        userAttributes[i].supported=false;
        userAttributes[i].unicode=false;
    }
    QStringList attrList=attrString.split("\n",QString::SkipEmptyParts);
    QString attrLine;
    foreach (attrLine, attrList)
    {
        QStringList attrWords=attrLine.split(":",QString::SkipEmptyParts);
        int index=0;
        if (attrWords[0]=="foto")
            index=FOTO;
        if (attrWords[0]=="uid")
            index=LOGIN;
        if (attrWords[0]=="uidnumber")
            index=UIDNUMBER;
        if (attrWords[0]=="firstname")
            index=FNAME;
        if (attrWords[0]=="lastname")
            index=LNAME;
        if (attrWords[0]=="birthday")
            index=BIRTHDAY;
        if (attrWords[0]=="group")
            index=PGROUP;
        if (attrWords[0]=="home")
            index=HOMEDIR;
        if (attrWords[0]=="shell")
            index=SHELL;
        userAttributes[index].supported=true;

        QStringList attrs=attrWords[1].split(";");
        QString uattr;
        foreach(uattr, attrs)
        {
            QStringList nameval=uattr.split("=");
            if (nameval[0]=="edit")
                userAttributes[index].editable=(nameval[1]!="0");
            if (nameval[0]=="unicode")
                userAttributes[index].unicode=(nameval[1]!="0");
        }
    }
}

void X2GouserAdminWindow::slotChangePass()
{
    bool ok;
    QString ps=QInputDialog::getText(this,tr("Set user password"), tr("User password:"), QLineEdit::Password, "", &ok);
    if (ok)
    {
        QString ps1=QInputDialog::getText(this,tr("Verify user password"), tr("Verify password:"), QLineEdit::Password, "", &ok);
        if (ps != ps1)
        {
            QMessageBox::critical(this,tr("Error"),tr("Passwords are not identical"));
            return;
        }
        if (ps1.length()<=0)
        {
            QMessageBox::critical(this,tr("Error"),tr("Password required"));
            return;
        }
        currentUser.newpassword=ps;
        slotUserChanged();
    }
}

void X2GouserAdminWindow::slotSaveGroupSelection()
{
    QStringList groups;
    for (int i=0; i<lselGroup->count(); ++i)
        groups<<lselGroup->item(i)->text();
    QSettings* st=centerInstance->getSettings(pluginName());
    st->setValue("preselectedgroups",groups);
    if (userAttributes[PGROUP].supported)
        st->setValue("primarygroup",cbPGroup->currentText());
    st->sync();
}


void X2GouserAdminWindow::initUserList(const QString& res)
{

    if (!userAttributes[LOGIN].supported)
    {
        treeWidget->hideColumn(COL_LOGIN);
        lFindLogin->hide();
        eFindLogin->hide();
        lUsername->hide();
    }
    if (!userAttributes[FNAME].supported)
    {
        treeWidget->hideColumn(COL_FNAME);
        lFname->hide();
        eFname->hide();
    }
    if (!userAttributes[LNAME].supported)
    {
        treeWidget->hideColumn(COL_LNAME);
        lLname->hide();
        eLname->hide();
    }
    if (!userAttributes[UIDNUMBER].supported)
    {
        treeWidget->hideColumn(COL_UIDNUMBER);
        cbMaxNumber->hide();
        cbMinNumber->hide();
        spMaxNumber->hide();
        spMinNumber->hide();
        lUid->hide();
        spUId->hide();
    }

    if (!userAttributes[FOTO].supported)
    {
        pbUserPic->hide();
        lChanheUserPic->hide();
    }

    if (!userAttributes[BIRTHDAY].supported)
    {
        lBirthday->hide();
        dateBirthday->hide();
    }

    if (!userAttributes[UIDNUMBER].supported)
    {
        lUid->hide();
        spUId->hide();
    }

    if (!userAttributes[PGROUP].supported)
    {
        lPGroup->hide();
        cbPGroup->hide();
    }
    if (!userAttributes[HOMEDIR].supported)
    {
        lHome->hide();
        eHome->hide();
    }
    if (!userAttributes[SHELL].supported)
    {
        lShell->hide();
        eShell->hide();
    }

    treeWidget->header()->setResizeMode ( QHeaderView::ResizeToContents );
    treeWidget->header()->setSortIndicator ( 1,Qt::AscendingOrder );

    int firstUserPos=res.indexOf("BEGIN_USERS\n");
    int lastUserPos=res.indexOf("END_USERS");
    QString users=res.mid(firstUserPos,lastUserPos-firstUserPos);
    users.replace("BEGIN_USERS\n","");


    if (cbTree->isChecked())
    {
        int firstGroupPos=res.indexOf("BEGIN_GROUPS\n");
        int lastGroupPos=res.indexOf("END_GROUPS");
        QString groups=res.mid(firstGroupPos,lastGroupPos-firstGroupPos);
        groups.replace("BEGIN_GROUPS\n","");

        treeWidget->setRootIsDecorated(true);
        treeWidget->setItemsExpandable(true);
        QStringList ulist=users.split("\n",QString::SkipEmptyParts);
        QString uentry;
        QList<struct user> listOfUsers;
        foreach(uentry, ulist)
        {
            QStringList udetails=uentry.split(":",QString::SkipEmptyParts);
            QString detail;
            struct user us;
            us.birthday="1970-01-01";
            foreach(detail,udetails)
            {
                QStringList fields=detail.split("=");
                QString name=fields[0];
                QString value=fields[1];
                if (name == "UID")
                    us.login=value;
                if (name == "FNAME")
                    us.fname=value;
                if (name == "LNAME")
                    us.lname=value;
                if (name == "UIDNUMBER")
                    us.uidn=value;
                if (name == "GID")
                    us.group=value;
                if (name == "HOME")
                    us.home=value;
                if (name == "SHELL")
                    us.shell=value;
                if (name == "BIRTHDAY")
                    us.birthday=value;
            }
            listOfUsers<<us;
        }

        QStringList glist=groups.split("\n",QString::SkipEmptyParts);
        QString gentry;
        foreach(gentry, glist)
        {
            QTreeWidgetItem* it=new QTreeWidgetItem ( treeWidget );
            it->setSizeHint(0,QSize(39,39));

            it->setIcon(0,QPixmap ( ":/png/group.png" ));

            QStringList gdetails=gentry.split(":",QString::SkipEmptyParts);
            QString gname;
            QStringList memberUids;
            QString gid;
            QString detail;
            foreach(detail,gdetails)
            {
                QStringList fields=detail.split("=");
                QString name=fields[0];
                QString value=fields[1];
                if (name == "NAME")
                    gname=value;
                if (name == "MEMBERS")
                    memberUids=value.split(";");
                if (name== "GID")
                    gid=value;
            }
            it->setFlags(it->flags()&~Qt::ItemIsSelectable);
            it->setChildIndicatorPolicy(QTreeWidgetItem::DontShowIndicatorWhenChildless);
            QString memberUid;
            int numOfItems=0;
            if (userAttributes[PGROUP].supported)
            {
                for (int i=0; i<listOfUsers.count(); ++i)
                {
                    if (listOfUsers[i].group==gid)
                    {
                        ++numOfItems;
                        QTreeWidgetItem* chit=new QTreeWidgetItem ( it);
                        setUserItem(chit,&listOfUsers[i]);
                    }
                }

            }
            foreach(memberUid, memberUids)
            {
                user us;
                us.login=memberUid;
                int ind=listOfUsers.indexOf(us);
                if (ind!=-1)
                {
                    ++numOfItems;
                    QTreeWidgetItem* chit=new QTreeWidgetItem ( it);
                    setUserItem(chit,&listOfUsers[ind]);
                }
            }
            it->setText(1,gname+"("+QString::number(numOfItems)+")");
        }
    }
    else
    {
        treeWidget->setRootIsDecorated(false);
        treeWidget->setItemsExpandable(false);

        QStringList ulist=users.split("\n",QString::SkipEmptyParts);
        QString uentry;
        foreach(uentry, ulist)
        {
            QTreeWidgetItem* it=new QTreeWidgetItem ( treeWidget );
            QStringList udetails=uentry.split(":",QString::SkipEmptyParts);
            QString detail;
            struct user us;
            us.birthday="1970-01-01";
            foreach(detail,udetails)
            {
                QStringList fields=detail.split("=");
                QString name=fields[0];
                QString value=fields[1];
                if (name == "UID")
                    us.login=value;
                if (name == "FNAME")
                    us.fname=value;
                if (name == "LNAME")
                    us.lname=value;
                if (name == "UIDNUMBER")
                    us.uidn=value;
                if (name == "GID")
                    us.group=value;
                if (name == "HOME")
                    us.home=value;
                if (name == "SHELL")
                    us.shell=value;
                if (name == "BIRTHDAY")
                    us.birthday=value;
            }
            setUserItem(it,&us);
        }
    }
    gbAllUsers->setEnabled(true);
    if (eFindLogin->text().length()>2 ||eFindName->text().length()>2)
        slotFindUser();
}

void X2GouserAdminWindow::setUserItem(QTreeWidgetItem* it, user* us)
{
    it->setSizeHint(0,QSize(39,39));
    it->setChildIndicatorPolicy(QTreeWidgetItem::DontShowIndicator);

    //add support for more attributes//save attributes in item data using userrole
    if (!userAttributes[FOTO].supported)
        it->setIcon(0,QPixmap ( ":/png/user.png" ));

    it->setText(COL_LOGIN,us->login);
    it->setText(COL_FNAME,us->fname);
    it->setText(COL_LNAME,us->lname);
    it->setText(COL_UIDNUMBER,us->uidn);
    it->setChildIndicatorPolicy(QTreeWidgetItem::DontShowIndicator);

    it->setData(COL_LOGIN, FOTOROLE,us->foto);
    it->setData(COL_LOGIN, LOGINROLE,us->login);
    it->setData(COL_LOGIN, UIDNUMBERROLE,us->uidn);
    it->setData(COL_LOGIN, FNAMEROLE,us->fname);
    it->setData(COL_LOGIN, LNAMEROLE,us->lname);
    it->setData(COL_LOGIN, HOMEDIRROLE,us->home);
    it->setData(COL_LOGIN, SHELLROLE,us->shell);
    it->setData(COL_LOGIN, PGROUPROLE,us->group);
    it->setData(COL_LOGIN, BIRTHDAYROLE,us->birthday);
}


void X2GouserAdminWindow::slotUsersList(QVariant& result)
{
    if (hidden)
        return;
    QString res=result.toString();
    if (!checkResult(res))
    {
        centerInstance->backToList();
        return;
    }
//     qDebug()<<res;
    int firstAttrPos=res.indexOf("BEGIN_USERATTR");
    int lastAttrPos=res.indexOf("END_USERATTR");
    QString attrs=res.mid(firstAttrPos, lastAttrPos-firstAttrPos);
    attrs.replace("BEGIN_USERATTR\n","");
    initUserAttr(attrs);
    initUserList(res);
    centerInstance->setStatus(tr("Ready"));

}


void X2GouserAdminWindow::slotGroupsOfUserList(QVariant& result)
{
    if (hidden)
        return;
    QString res=result.toString();
//     qDebug()<<res;
    if (!checkResult(res))
    {
        centerInstance->backToList();
        return;
    }
    QStringList groupEntries=res.split("\n",QString::SkipEmptyParts);
    QString entry;
    currentUser.groups.clear();
    lselGroup->clear();
    lunselGroup->clear();
    QStringList preselGroups;
    QString primGroup;
    QStringList allGroups;
    if (currentUser.newUser)
    {
        QSettings* st=centerInstance->getSettings(pluginName());
        preselGroups=st->value("preselectedgroups").toStringList();
        if (userAttributes[PGROUP].supported)
            primGroup=st->value("primarygroup","users").toString();
    }
    foreach(entry, groupEntries)
    {
        QStringList elements=entry.split(":",QString::SkipEmptyParts);
        QString name=elements[0];
        QString member=elements[1];
        QString gid;
        if (userAttributes[PGROUP].supported)
        {
            gid=elements[2];
        }
        if (!currentUser.newUser)
        {
            if (userAttributes[PGROUP].supported)
            {
                allGroups<<name;
            }
            if (member == "0")
            {
                if (userAttributes[PGROUP].supported)
                {
                    if (gid==currentUser.pgroup)
                    {
                        primGroup=name;
                    }
                    else
                        lunselGroup->addItem(name);
                }
                else
                    lunselGroup->addItem(name);
            }
            else
            {
                lselGroup->addItem(name);
                currentUser.groups<<name;
            }
        }
        else
        {
            if (userAttributes[PGROUP].supported)
            {
                allGroups<<name;
            }
            if (preselGroups.contains(name))
                lselGroup->addItem(name);
            else
            {
                if (name!=primGroup)
                    lunselGroup->addItem(name);
            }
        }
    }
    if (userAttributes[PGROUP].supported)
    {
        cbPGroup->clear();
        allGroups.sort();
        int index=allGroups.indexOf(primGroup);
        cbPGroup->insertItems(0,allGroups);
        cbPGroup->setCurrentIndex(index);
        currentUser.pgroup=primGroup;
    }
    lselGroup->sortItems();
    lunselGroup->sortItems();
    setUserViewEnabled(true);
    gbAllUsers->setEnabled(true);
    centerInstance->setStatus(tr("Ready"));
    changed=false;
    pbApply->setEnabled(false);
}


void X2GouserAdminWindow::slotFindUser()
{
    QTreeWidgetItemIterator it(treeWidget);
    QString slogin=eFindLogin->text();
    QString sname=eFindName->text();
    bool searchlogin=(slogin.length()>2);
    bool searchname=(sname.length()>2);
    while (*it)
    {
        if ((*it)->childIndicatorPolicy()==QTreeWidgetItem::DontShowIndicatorWhenChildless)
        {
            ++it;
            continue;
        }
        QString login=(*it)->data(COL_LOGIN,LOGINROLE).toString();
        QString name=(*it)->data(COL_LOGIN,FNAMEROLE).toString()+(*it)->data(COL_LOGIN,LNAMEROLE).toString();
        (*it)->setHidden(false);
        if (userAttributes[UIDNUMBER].supported)
        {
            int uid=(*it)->data(COL_LOGIN,UIDNUMBERROLE).toString().toInt();
            if (cbMaxNumber->isChecked() && spMaxNumber->value()<uid)
                (*it)->setHidden(true);
            if (cbMinNumber->isChecked() && spMinNumber->value()>uid)
                (*it)->setHidden(true);
        }
        if (searchlogin && (! login.contains(slogin,Qt::CaseInsensitive)))
            (*it)->setHidden(true);
        else if (searchname && (!name.contains(sname,Qt::CaseInsensitive)))
            (*it)->setHidden(true);
        else if (cbTree->isChecked())
            (*it)->parent()->setExpanded(true);
        ++it;
    }
}
bool X2GouserAdminWindow::stringIsAscii(const QString& str)
{
    QByteArray ba=str.toAscii();
    for (int i=0; i<ba.size(); ++i)
        if (!isascii(ba[i]))
            return false;
    return true;
}

void X2GouserAdminWindow::slotAddUser()
{
    currentUser.newUser=true;
    treeWidget->clearSelection();
    slotUserSelected(0,0);
}

void X2GouserAdminWindow::slotLoginChanged(const QString& login)
{
    eHome->setText("/home/"+login);
}

bool X2GouserAdminWindow::isAttributeUnic(int role, QString value)
{
    QTreeWidgetItemIterator it(treeWidget);
    while (*it)
    {
        if ((*it)->childIndicatorPolicy()==QTreeWidgetItem::DontShowIndicatorWhenChildless)
        {
            ++it;
            continue;
        }
        if ((*it)->data(COL_LOGIN,role).toString()==value)
            return false;
        ++it;
    }
    return true;
}

int X2GouserAdminWindow::findFreeUid()
{
    QVector<int> uids;
    QTreeWidgetItemIterator it(treeWidget);
    while (*it)
    {
        if ((*it)->childIndicatorPolicy()==QTreeWidgetItem::DontShowIndicatorWhenChildless)
        {
            ++it;
            continue;
        }
        uids<<(*it)->data(COL_LOGIN,UIDNUMBERROLE).toString().toInt();
        ++it;
    }
    qSort(uids);
    int uid=1000;
    foreach (int id, uids)
    {
        if (id > uid)
            return uid;
        if (id==uid)
            ++uid;
    }
    return uid;
}


void X2GouserAdminWindow::slotUserSelected(QTreeWidgetItem* item , int col)
{
    if (col ==-1)
        return;
    if (item&&item->childIndicatorPolicy()==QTreeWidgetItem::DontShowIndicatorWhenChildless)
        return;
//     qDebug()<< "user selected";
    if (changed)
    {
        int res=applyQestion();
        if (res==QMessageBox::Cancel)
            return;
        if (res==QMessageBox::Yes)
        {
            slotApply();
            return;
        }
    }
    dateBirthday->setDate(QDate(1970,1,1));
    if (item)
    {
        eLogin->setText(item->data(COL_LOGIN,LOGINROLE).toString());
        lLogin->hide();
        eLogin->hide();
        lUsername->show();
        currentUser.newUser=false;
        //add support for other atributes
        currentUser.password=currentUser.newpassword=QString::null;
        currentUser.item=item;
        if (userAttributes[FNAME].supported)
        {
            eFname->setText(item->data(COL_LOGIN,FNAMEROLE).toString());
            eFname->setReadOnly(!userAttributes[FNAME].editable);
            currentUser.fname=item->data(COL_LOGIN,FNAMEROLE).toString();
        }
        if (userAttributes[LNAME].supported)
        {
            eLname->setText(item->data(COL_LOGIN,LNAMEROLE).toString());
            eLname->setReadOnly(!userAttributes[LNAME].editable);
            currentUser.lname=item->data(COL_LOGIN,LNAMEROLE).toString();
        }
        if (userAttributes[LOGIN].supported)
        {
            lUsername->setText(item->data(COL_LOGIN,LOGINROLE).toString());
            currentUser.login=item->data(COL_LOGIN,LOGINROLE).toString();
        }
        if (userAttributes[UIDNUMBER].supported)
        {
            spUId->setValue(item->data(COL_LOGIN,UIDNUMBERROLE).toString().toInt());
            spUId->setEnabled(userAttributes[UIDNUMBER].editable);
            currentUser.uidnumber=item->data(COL_LOGIN,UIDNUMBERROLE).toString();
        }
        if (userAttributes[HOMEDIR].supported)
        {
            eHome->setText(item->data(COL_LOGIN,HOMEDIRROLE).toString());
            eHome->setReadOnly(!userAttributes[HOMEDIR].editable);
            currentUser.homedir=item->data(COL_LOGIN,HOMEDIRROLE).toString();
        }
        if (userAttributes[SHELL].supported)
        {
            eShell->setText(item->data(COL_LOGIN,SHELLROLE).toString());
            eShell->setReadOnly(!userAttributes[SHELL].editable);
            currentUser.shell=item->data(COL_LOGIN,SHELLROLE).toString();
        }
        if (userAttributes[PGROUP].supported)
        {
            currentUser.pgroup=item->data(COL_LOGIN,PGROUPROLE).toString();
            cbPGroup->setEnabled(userAttributes[PGROUP].editable);
        }
        if (userAttributes[BIRTHDAY].supported)
        {
            dateBirthday->setDate(item->data(COL_LOGIN,BIRTHDAYROLE).toDate());
            dateBirthday->setReadOnly(!userAttributes[BIRTHDAY].editable);
            currentUser.birthday=item->data(COL_LOGIN,BIRTHDAYROLE).toDate();
        }

    }
    else
    {
        //add support for other attributes
        if (adminMode)
        {
            if (userAttributes[FNAME].supported)
            {
                eFname->setText(QString::null);
            }
            if (userAttributes[LNAME].supported)
            {
                eLname->setText(QString::null);
            }
            if (userAttributes[UIDNUMBER].supported)
            {
                spUId->setValue(findFreeUid());
                spUId->setEnabled(true);
            }
            if (userAttributes[HOMEDIR].supported)
            {
                eHome->setText("/home/login");
                eHome->setReadOnly(false);
            }
            if (userAttributes[SHELL].supported)
            {
                eShell->setText("/bin/bash");
                eShell->setReadOnly(false);
            }
            if (userAttributes[BIRTHDAY].supported)
            {
                dateBirthday->setDate(QDate());
                eShell->setReadOnly(false);
            }
            if (userAttributes[PGROUP].supported)
            {
                cbPGroup->setEnabled(true);
            }

            lUsername->hide();
            currentUser.login="";
            eLogin->setText("login");
            eLogin->show();
            lLogin->show();
            slotUserChanged();
        }
        else
        {
            /*            if (userAttributes[FNAME].supported)
                        {
                            eFname->setText(QString::null);
                            eFname->setEnabled(false);
                        }
                        if (userAttributes[LNAME].supported)
                        {
                            eLname->setText(QString::null);
                            eLname->setEnabled(false);
                        }
                        lUsername->setText(centerInstance->getLogin());
                        QVariant var;
                        slotGroupsOfUserList(var);*/
            return;
        }
    }
    QVariantList args;
    args<<centerInstance->getLogin()<<centerInstance->getPassword()<<currentUser.login;

    rpc->call("x2goadmin.getGroupsOfUser",args, this, SLOT(slotGroupsOfUserList(QVariant&)),
              this, SLOT(slotRpcFault(int, const QString&)));

    setUserViewEnabled(false);
    gbAllUsers->setEnabled(false);
    lselGroup->clear();
    lunselGroup->clear();
    centerInstance->setStatus(tr("Retrieving information from server, please wait..."));
    changed=false;
    pbApply->setEnabled(false);
}

void X2GouserAdminWindow::slotAddGroup()
{
    for (int i=lunselGroup->count()-1; i >= 0; --i)
    {
        if (lunselGroup->item(i)->isSelected())
        {
            lselGroup->addItem(lunselGroup->item(i)->text());
            delete lunselGroup->takeItem(i);
            changed=true;
            pbApply->setEnabled(true);
        }
    }
    lselGroup->sortItems();
    lunselGroup->sortItems();
}

void X2GouserAdminWindow::slotRemoveGroup()
{
    for (int i=lselGroup->count()-1; i >= 0; --i)
    {
        if (lselGroup->item(i)->isSelected())
        {
            lunselGroup->addItem(lselGroup->item(i)->text());
            delete lselGroup->takeItem(i);
            changed=true;
            pbApply->setEnabled(true);
        }
    }
    lselGroup->sortItems();
    lunselGroup->sortItems();

}

void X2GouserAdminWindow::slotUserChanged()
{
    changed=true;
    pbApply->setEnabled(true);
}

void X2GouserAdminWindow::slotApply()
{
    //add support for other atributes
    QVariantList args;
    QString fname,lname,login,uidnumber,pgroup,homedir,shell;
    QDate birthday;

    login=eLogin->text();
    if (currentUser.newUser)
        currentUser.login=login;
    args<<centerInstance->getLogin()<<centerInstance->getPassword()<<currentUser.login;
    if (currentUser.password!=currentUser.newpassword)
    {
        args<<"password:"+currentUser.newpassword;
    }
    if (userAttributes[FNAME].supported)
    {
        fname=eFname->text();
        if ( (!userAttributes[FNAME].unicode && !stringIsAscii(fname) )|| fname.length()<=0)
        {
            QMessageBox::critical(this,tr("Error"),tr("Invalid input: \"First name\""));
            return;
        }
        if (currentUser.fname != fname || currentUser.newUser)
        {
            args<<"fname:"+fname;
        }
    }
    if (userAttributes[BIRTHDAY].supported)
    {
        birthday=dateBirthday->date();
        if (currentUser.birthday != birthday || currentUser.newUser)
        {
            args<<"birthday:"+birthday.toString("yyyy-MM-dd");
        }
    }
    if (userAttributes[LNAME].supported)
    {
        lname=eLname->text();
        if ( (!userAttributes[LNAME].unicode && !stringIsAscii(lname) )|| lname.length()<=0)
        {
            QMessageBox::critical(this,tr("Error"),tr("Invalid input: \"Last name\""));
            return;
        }
        if (currentUser.lname != lname || currentUser.newUser)
        {
            args<<"lname:"+lname;
        }
    }
    if (userAttributes[LOGIN].supported)
    {
        if (( !userAttributes[LOGIN].unicode && !stringIsAscii(login))||login.length()<=0 )
        {
            QMessageBox::critical(this,tr("Error"),tr("Invalid input: \"Login\""));
            return;
        }
        if (!isAttributeUnic(LOGINROLE,login)&&currentUser.newUser)
        {
            QMessageBox::critical(this,tr("Error"),tr("User already exists: ")+login);
            return;
        }
        if (currentUser.login != login)
        {
            args<<"login:"+login;
        }
    }


    if (userAttributes[UIDNUMBER].supported)
    {
        //check uidnumber
        uidnumber=QString::number(spUId->value());
        if (currentUser.uidnumber != uidnumber || currentUser.newUser)
        {
            if (!isAttributeUnic(UIDNUMBERROLE,uidnumber))
            {
                QMessageBox::critical(this,tr("Error"),tr("User with this UID already exists: ")+uidnumber);
                return;
            }
            args<<"uidnumber:"+uidnumber;
        }
    }



    if (userAttributes[PGROUP].supported)
    {
        pgroup=cbPGroup->currentText();
        if (currentUser.pgroup != pgroup || currentUser.newUser)
        {
            args<<"primarygroup:"+pgroup;
        }
    }

    if (userAttributes[HOMEDIR].supported)
    {
        homedir=eHome->text();
        if ( (!userAttributes[HOMEDIR].unicode && !stringIsAscii(homedir) )|| homedir.length()<=0)
        {
            QMessageBox::critical(this,tr("Error"),tr("Invalid input: \"Home directory\""));
            return;
        }
        if (currentUser.homedir != homedir || currentUser.newUser)
        {
            args<<"homedirectory:"+homedir;
        }
    }

    if (userAttributes[SHELL].supported)
    {
        shell=eShell->text();
        if ( (!userAttributes[SHELL].unicode && !stringIsAscii(shell) )|| shell.length()<=0)
        {
            QMessageBox::critical(this,tr("Error"),tr("Invalid input: \"Login shell\""));
            return;
        }
        if (currentUser.shell != shell || currentUser.newUser)
        {
            args<<"loginshell:"+shell;
        }
    }

    QStringList newGroups;
    QStringList selGroups;
    for (int i=0; i<lselGroup->count(); ++i)
    {
        selGroups<<lselGroup->item(i)->text();
        if (!currentUser.groups.contains(lselGroup->item(i)->text()))
            newGroups<<lselGroup->item(i)->text();
    }
    if (newGroups.count())
        args<<"newgroups:"+newGroups.join(";");
    QStringList obsoleteGroups;
    for (int i=0; i<currentUser.groups.count(); ++i)
    {
        if (!selGroups.contains(currentUser.groups[i]))
            obsoleteGroups<<currentUser.groups[i];
    }
    if (obsoleteGroups.count())
        args<<"obsoletegroups:"+obsoleteGroups.join(";");

    if (args.count()>3)
    {
        if (currentUser.newUser)
            rpc->call("x2goadmin.addUser",args, this, SLOT(slotUsersModified(QVariant&)),
                      this, SLOT(slotRpcFault(int, const QString&)));
        else
            rpc->call("x2goadmin.modifyUser",args, this, SLOT(slotUsersModified(QVariant&)),
                      this, SLOT(slotRpcFault(int, const QString&)));
        currentUser.password=currentUser.newpassword;
        currentUser.login=login;
        currentUser.fname=fname;
        currentUser.lname=lname;
        currentUser.pgroup=pgroup;
        currentUser.uidnumber=uidnumber;
        centerInstance->setStatus(tr("Retrieving information from server, please wait..."));
    }
    else
    {
        QVariant result;
        result="OK";
        slotUsersModified(result);
    }
    gbAllUsers->setEnabled(false);
    setUserViewEnabled(false);
}

void X2GouserAdminWindow::slotUsersModified(QVariant& result)
{
//when ready  check if closing, set closing to false and close plugin
    if (hidden)
        return;
    centerInstance->setStatus(tr("Ready"));

    gbAllUsers->setEnabled(true);
    setUserViewEnabled(true);
    QString res=result.toString();
//     qDebug()<<res;
    if (!checkResult(res))
    {
        centerInstance->backToList();
        return;
    }
    if (closing)
    {
        closing=false;
        changed=false;
        centerInstance->backToList();
        return;
    }
    changed=false;
    pbApply->setEnabled(false);
    slotGetUsers();
}

void X2GouserAdminWindow::slotRmUsers()
{
    if (changed)
    {
        int res=applyQestion();
        if (res==QMessageBox::Cancel)
            return;
        if (res==QMessageBox::Yes)
        {
            slotApply();
            return;
        }
    }
    QStringList users;
    QTreeWidgetItemIterator it(treeWidget);
    while (*it)
    {
        if ((*it)->isSelected())
            users<<(*it)->text(COL_LOGIN);
        ++it;
    }
    if (!users.count())
        return;

    if (QMessageBox::warning(this,tr("Warning"),tr("Delete selected user(s)"),QMessageBox::Yes, QMessageBox::No)!=QMessageBox::Yes)
        return;

    QVariantList args;
    args<<centerInstance->getLogin()<<centerInstance->getPassword()<<users.join(";");

    rpc->call("x2goadmin.removeUsers",args, this, SLOT(slotUsersRemoved(QVariant&)),
              this, SLOT(slotRpcFault(int, const QString&)));

    gbAllUsers->setEnabled(false);
    setUserViewEnabled(false);
    centerInstance->setStatus(tr("Retrieving information from server, please wait..."));

}

void X2GouserAdminWindow::slotUsersRemoved(QVariant& result)
{
    if (hidden)
        return;
    centerInstance->setStatus(tr("Ready"));

    gbAllUsers->setEnabled(true);
    setUserViewEnabled(true);
    QString res=result.toString();
    qDebug()<<res;
    if (!checkResult(res))
    {
        centerInstance->backToList();
        return;
    }
    startPlugin();
}

void X2GouserAdminWindow::slotChangePrimaryGroup(const QString& group)
{
    if (!gbUser->isVisible() || !gbUser->isEnabled())
        return;
    QList<QListWidgetItem *> items=lselGroup->findItems(group,Qt::MatchExactly);
    if (items.count()==1)
    {
        delete items[0];
    }
    items=lunselGroup->findItems(group,Qt::MatchExactly);
    if (items.count()==1)
    {
        delete items[0];
    }
    for (int i=0; i<cbPGroup->count(); ++i)
    {
        QString grp=cbPGroup->itemText(i);
        if (grp==group)
            continue;
        if (lselGroup->findItems(grp,Qt::MatchExactly).count()==0 &&
                lunselGroup->findItems(grp,Qt::MatchExactly).count()==0)
        {
            lunselGroup->addItem(grp);
        }
    }
    lselGroup->sortItems();
    lunselGroup->sortItems();
}

void X2GouserAdminWindow::setUserViewEnabled(bool enable)
{
    gbUser->setEnabled(enable);
    if (enable)
        setUserViewVisible(true);
}

void X2GouserAdminWindow::setUserViewVisible(bool visible)
{
    gbUser->setVisible(visible);
    lSelect->setVisible(!visible);
}


Q_EXPORT_PLUGIN2(x2gouseradmin, X2GouserAdminWindow)
