/*
    Part of X2GoAdminCenter.
    Copyright (C) 2011-2015 Oleksandr Shneyder<oleksandr.shneyder@obviously-nice.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef X2GOUSERADMINWINDOW_H
#define X2GOUSERADMINWINDOW_H

#include <x2goadminInterface.h>

#include "ui_x2gouseradminui.h"


#define VERSION "0.0.0.1"

#define APPICON QPixmap(":/png/x2gouser.png")
#define USERATTR SHELL+1

#include <maiaXmlRpcClient.h>
struct user {
    QPixmap foto;
    QString birthday,group,home,shell,login,fname,lname,uidn;
    bool operator == (const struct user& us)
    {
        return (us.login==login);
    }
};

class X2GouserAdminWindow :public QWidget, public X2goAdminPlugin, private Ui::X2GoUserAdminUI
{
    Q_OBJECT
    Q_INTERFACES(X2goAdminPlugin)
public:
    X2GouserAdminWindow();
    ~X2GouserAdminWindow();
    QString pluginName() {
        return NAME;
    }
    QString description() {
        return DESCRIPTION;
    }
    QString about() {
        return ABOUT;
    }
    QPixmap pluginPic() {
        return APPICON;
    }
    QWidget* widget() {
        return this;
    }
    bool closePlugin();
    void setCenterInstance(CenterInterface *obj);
    void startPlugin();

private:

    enum {FOTO,LOGIN,UIDNUMBER,FNAME,LNAME,BIRTHDAY,PGROUP,HOMEDIR,SHELL};
    enum {COL_FOTO,COL_LOGIN,COL_FNAME,COL_LNAME,COL_UIDNUMBER};
    enum {FOTOROLE=Qt::UserRole+11, LOGINROLE,UIDNUMBERROLE,FNAMEROLE,LNAMEROLE,
          BIRTHDAYROLE,PGROUPROLE,HOMEDIRROLE,SHELLROLE
         };
    struct userattr
    {
        bool supported;
        bool editable;
        bool unicode;
    };
    struct {
        QPixmap foto;
        QString login,fname,lname,pgroup,homedir,shell,password, newpassword, uidnumber;
        QDate birthday;
        QStringList groups;
        QTreeWidgetItem *item;
        bool newUser;
    }currentUser;

    bool checkResult(const QString& result);
    void initUserAttr(const QString& attrString);
    void initUserList(const QString& users);
    bool stringIsAscii(const QString& str);
    int applyQestion();
    void installTranslator();
    void setUserViewEnabled(bool enable);
    void setUserViewVisible(bool visible);
    int findFreeUid();
    bool isAttributeUnic(int role, QString value);
    void setUserItem(QTreeWidgetItem* it, struct user* us);
    CenterInterface* centerInstance;
    MaiaXmlRpcClient* rpc;
    bool changed;
    bool hidden;
    bool closing;
    bool adminMode;
    userattr userAttributes[USERATTR];
    QString NAME;
    QString DESCRIPTION;
    QString DEVELOPERS;
    QString TRANSLATORS;
    QString ABOUT;


private slots:
    void slotHandleSslErrors(QNetworkReply *reply, const QList<QSslError> &errors);
    void slotUsersList(QVariant& result);
    void slotUsersModified(QVariant& result);
    void slotGroupsOfUserList(QVariant& result);
    void slotRpcFault(int, const QString& errMessage);
    void slotFindUser();
    void slotUserSelected(QTreeWidgetItem*,int);
    void slotChangePass();
    void slotAddGroup();
    void slotRemoveGroup();
    void slotUserChanged();
    void slotApply();
    void slotRmUsers();
    void slotUsersRemoved(QVariant& result);
    void slotAddUser();
    void slotSaveGroupSelection();
    void slotGetUsers();
    void slotChangePrimaryGroup(const QString& group);
    void slotLoginChanged(const QString& login);
};

#endif // X2GOUSERADMINWINDOW_H
